#ifndef LIERO_MAIN_HPP
#define LIERO_MAIN_HPP

#ifdef _TINSPIRE
#include <os.h>
#include <nucleus.h>
#include <libndls.h>
#include <syscall-list.h>
#include <syscall.h>
#include <nspireio/nspireio.h>
#endif

#include <SDL/SDL.h>

#include "gfx.hpp"
#include "sfx.hpp"
#include "sys.hpp"
#include "game.hpp"
#include "viewport.hpp"
#include "worm.hpp"
#include "reader.hpp"
#include "filesystem.hpp"
#include "text.hpp"
#include "keys.hpp"
#include "constants.hpp"
#include "math.hpp"
#include "console.hpp"
#include "platform.hpp"

#include <iostream>
#include <ctime>
#include <exception>

//#undef main

int gameEntry(int argc, char* argv[])
try
{
#ifdef _TINSPIRE
	enable_relative_paths(argv);
#endif
	// TODO: Better PRNG seeding
	gfx.rand.seed(0);
	
	bool exeSet = false;
	game.loadPowerlevelPalette = true;

	exeSet = true;
	
	char datadir[192];
	char *buf = reinterpret_cast<char*>(malloc(PATH_MAX));
	syscall<e_NU_Current_Dir, int>("A:\\", buf);
	
	sprintf(datadir, "%sdata\\",buf);
	setLieroDataRoot(datadir);
	/*setConfigRoot();*/

#ifdef SOUND_ENABLED
	SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO);
#else
	SDL_Init(SDL_INIT_VIDEO );
#endif
	
/*
	char buf[256];
	std::cout << SDL_VideoDriverName(buf, 256) << std::endl;
*/
	

	game.texts.loadFromEXE();
	initKeys();
	game.rand.seed(0);
	
	loadConstants();
	
	loadTablesFromEXE();
	
	
	gfx.font.loadFromEXE();
	gfx.loadPalette();
	gfx.loadMenus();
	
	gfx.loadGfx();
	game.loadMaterials();
	game.loadWeapons();
	game.loadTextures();
	
	game.loadOthers();
	
	sfx.init();


#ifdef SOUND_ENABLED
	SDL_version const* mixerVer = Mix_Linked_Version();
#else
	SDL_version const* mixerVer = NULL;
#endif
	
	/*sfx.loadFromSND();*/
	
	gfx.init();
	
	game.settingsFile = "LIERO";
	
	if(!fileExists(lieroOPT)) // NOTE: Liero doesn't seem to use the contents of LIERO.OPT for anything useful
	{
		game.settings = Settings();
		game.saveSettings();
	}
	else
	{
		FILE* f = fopen(lieroOPT.c_str(), "rb");
		std::size_t len = fileLength(f);
		if(len > 255) len = 255;
		char buf[256];
		fread(buf, 1, len, f);
		game.settingsFile.assign(buf, len);
		
		rtrim(game.settingsFile);
		if(!game.loadSettings())
		{
			game.settingsFile = "LIERO";
			game.settings = Settings();
			game.saveSettings();
		}
		fclose(f);
	}
	
	game.initGame();
	
	gfx.mainLoop();
	
	
	
	game.settingsFile = "LIERO";

#ifdef NSPIRE
	game.settings.save(joinPath(lieroDataRoot, "LIERO.DAT.tns"));
#else
	game.settings.save(joinPath(lieroDataRoot, "LIERO.DAT"));
#endif
	
	FILE* f = fopen(lieroOPT.c_str(), "wb");
	fwrite(game.settingsFile.data(), 1, game.settingsFile.size(), f);
	fputc('\r', f);
	fputc('\n', f);
	fclose(f);
	
	closeAllCachedFiles();
	
	SDL_Quit();
	
	return 0;
}
catch(std::exception& ex)
{
	SDL_Quit();
	Console::setAttributes(0x2f);
	Console::writeLine(std::string("EXCEPTION: ") + ex.what());
	Console::waitForAnyKey();
	return 1;
}

#endif // LIERO_MAIN_HPP

